package eu.etaxonomy.taxeditor.prototype2.controller;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * For drag and drop - a drag widget with this transfer type 
 *  can only be dropped onto a drop widget with the same type.
 *    
 * @author p.ciardelli
 *
 */
public class SynonymRelationshipTransfer extends ByteArrayTransfer {

	private static final String RELATIONTYPENAME = "relation_type";
	private static final int RELATIONTYPEID = registerType (RELATIONTYPENAME);		
	private static final SynonymRelationshipTransfer INSTANCE = new SynonymRelationshipTransfer();
	
	/*
	 * The object associated with this transfer event
	 */
	private SynonymRelationship synonymRelationship;
		
	/*
	 * Returns the singleton
	 */
	public static SynonymRelationshipTransfer getInstance() {
		return INSTANCE;
	}
	/*
	 * Set transfer data for local use
	 */
	public void setSynonymRelationship(SynonymRelationship synonymRelationship) {
		this.synonymRelationship = synonymRelationship;
	}
	/*
	 * Returns the local transfer data
	 */
	public SynonymRelationship getSynonymRelationship() {
		return synonymRelationship;
	}
	
	/* 
	 * The type ID is used to identify this transfer
	 */
	@Override
	protected int[] getTypeIds() {
		return new int[] { RELATIONTYPEID };
	}

	@Override
	protected String[] getTypeNames() {
		return new String[] { RELATIONTYPENAME } ;
	}

	@Override
	protected void javaToNative(Object object, TransferData transferData) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See nativeToJava below
		byte[] check = RELATIONTYPENAME.getBytes();
		super.javaToNative(check, transferData);
	}

	@Override
	protected Object nativeToJava(TransferData transferData) {
		Object result = super.nativeToJava(transferData);
		if (isInvalidNativeType(result)) {
			throw new RuntimeException(); //$NON-NLS-1$
		}
		return synonymRelationship;
	}
	
	private boolean isInvalidNativeType(Object result) {
		// No encoding needed since this is a hardcoded string read and written
		// in the same process.
		// See javaToNative above
		return !(result instanceof byte[])
				|| !RELATIONTYPENAME.equals(new String((byte[]) result));
	}
}
