package eu.etaxonomy.taxeditor.prototype2.view;

import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionOpenNameEditor;

public class RecentNamesTableViewer extends TableViewer {

	public RecentNamesTableViewer(Composite parent) {
		super(parent, SWT.H_SCROLL | SWT.V_SCROLL);
		this.getTable().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		// Add content provider
		ObservableListContentProvider rnViewerContentProviderList = new ObservableListContentProvider();
		this.setContentProvider(rnViewerContentProviderList);
		
		// Label provider that listens for changes to name cache
		IObservableMap[] rnViewerLabelProviderMaps = BeansObservables.observeMaps(rnViewerContentProviderList.
				getKnownElements(), Taxon.class, new String[]{"name"});
		this.setLabelProvider(new ObservableMapLabelProvider(rnViewerLabelProviderMaps) {
			
			/*
			 * JFace databinding syntax makes it to difficult to retrieve
			 * 	Taxon.getName().getNameCache, so override function that returns
			 *  label
			 * 
			 *  (non-Javadoc)
			 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnText(java.lang.Object, int)
			 */
			public String getColumnText(Object element, int columnIndex) {
				return ((Taxon) element).getName().getTitleCache();
			}			
		});
		
		// Listens for new taxa opened for editing
		this.setInput(Activator.getDefault().getObservableRecentNamesList());
		
		// On double click, open name editor
		this.addDoubleClickListener(new IDoubleClickListener(){

			public void doubleClick(DoubleClickEvent event) {								
				Taxon taxon = null;
				try {
					taxon = (Taxon) ((StructuredSelection)event.getSelection()).getFirstElement();
				}catch (Exception e){
					e.printStackTrace();
					taxon = null;
				}
				new ActionOpenNameEditor(taxon).run();
			}
		});
	}
}