/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.operations.description.RemoveElementOperation;
import eu.etaxonomy.taxeditor.propertysheet.description.DescriptionElementPropertySource;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class DescriptionElementComposite extends GroupedComposite {
	private static final Logger logger = Logger.getLogger(DescriptionElementComposite.class);

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public static final String TEXT_DATA = "text_data";
	public static final String COMMON_TAXON_NAME = "common_taxon_name";
	public static final String DISTRIBUTION = "distribution";

	/**
	 * ************ FONTS ************
	 */
	public static final Font ELEMENT_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.SYNONYM_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image BLACK_SQUARE_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);

	private DescriptionElementBase element;

	private IUndoContext undoContext;

	private static final String EMPTY_NAME_PROMPT = "Click to edit element composite";

	public DescriptionElementComposite(Composite parent, IManagedForm managedForm, 
			DescriptionElementBase element) {
		super(parent, managedForm);
		
		this.element = element;
		
		this.undoContext = EditorController.getUndoContext(taxon);
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();
		
		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIsDraggable(false);
		setIcon(BLACK_SQUARE_ICON);
		setFont(getViewerFont());
		
		createMenu();
		
		initTextViewer();
	}
	
	private void initTextViewer() {
		
		String text = null;
		if (element instanceof TextData) {
			text = ((TextData) element).getText(Language.DEFAULT());
		}
		if (text == null) {
			text = "";
		}
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			textViewer.getTextWidget().setText(text);
		}
	}

//	protected String getEmptyTextPrompt() {
//		return EMPTY_NAME_PROMPT;
//	}
	
	public DescriptionElementBase getElement() {
		return element;
	}
	
	protected void parse(String text) {
		
		if (getElement() instanceof TextData) {
			((TextData) getElement()).putText(text, Language.DEFAULT());
		}
		
		setDirty(true);
	}
	
	private void createMenu() {
		ContextMenu contextMenu = createContextMenu();
		
		// Delete this description element
		String text = "Delete description element";
		ImageDescriptor image = TaxEditorPlugin.getDefault()
				.getImageDescriptor(ITaxEditorConstants.ACTIVE_DELETE_ICON);
		contextMenu.addAction(new Action(text, image){
						
			public void run() {
				IUndoableOperation operation = new RemoveElementOperation
						(this.getText(), undoContext, taxon, element);
				
				GlobalController.executeOperation(operation);
			}
		});
	}

	
	public IPropertySource getPropertySource() {
		return new DescriptionElementPropertySource(getElement());
	}

	@Override
	protected Font getViewerFont() {
		return ELEMENT_FONT;
	}
	
	public Object getData () {
		return getElement();
	}
}