/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.taxeditor.ITaxEditorConstants;

/** 
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 */
public class InitNomenclaturalCodePrefDialog extends Dialog {
	private static final Logger logger = Logger
			.getLogger(InitNomenclaturalCodePrefDialog.class);
	
	private Button icbnButton;
	private Button icznButton;

	/**
	 * If the user makes no chance, or cancels, preferred nom. code will be set to the default
	 */
	private String preferredCode = ITaxEditorConstants.DEFAULT_CODE_PREFERENCE;

	/**
	 * Create the dialog
	 * @param parentShell
	 */
	public InitNomenclaturalCodePrefDialog(Shell parentShell) {
		super(parentShell);
	}
	
	/**
	 * Create contents of the dialog
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {

		Composite container = (Composite) super.createDialogArea(parent);

		// Create label with description of dialog
		final Label chooseANomenclaturalLabel = new Label(container, SWT.WRAP);
		chooseANomenclaturalLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		chooseANomenclaturalLabel.setText("Welcome to the EDIT Taxonomic Editor.\n\nBefore proceeding, please choose a nomenclatural code. This code will be used to create new names and to parse name text.");

		// Create composite to show radio buttons
		final Composite buttonsComposite = new Composite(container, SWT.NONE);
		final GridData gd_buttonsComposite = new GridData();
		gd_buttonsComposite.horizontalIndent = 20;
		buttonsComposite.setLayoutData(gd_buttonsComposite);
		buttonsComposite.setLayout(new GridLayout());

		// Create ICBN radio button
		icbnButton = new Button(buttonsComposite, SWT.RADIO);
		icbnButton.setText("International Code of Botanical Nomenclature (ICBN)");
		icbnButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setPreferredCode(ITaxEditorConstants.CODE_PREFERENCE_ICBN);
			}
		});

		// Create ICZN radio button
		icznButton = new Button(buttonsComposite, SWT.RADIO);
		icznButton.setText("International Code of Zoological Nomenclature (ICZN)");
		icznButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				setPreferredCode(ITaxEditorConstants.CODE_PREFERENCE_ICZN);
			}
		});
	
		// More dialog text		
		final Label chooseANomenclaturalLabel_1 = new Label(container, SWT.WRAP);
		chooseANomenclaturalLabel_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		chooseANomenclaturalLabel_1.setText("If you hit \"Cancel\", & the International Code of Botanical Nomenclature (ICBN) will be set as your default nomenclatural code.\n\nYou can change the botanical code at any time in the \"Preferences\" menu.");
		//
		return container;
	}

	protected void setPreferredCode(String preferredCode) {
		this.preferredCode = preferredCode;
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 375);
	}
	
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Choose a nomenclatural code");
	}
		
	public boolean close() {
		
		// Save preferred nomenclatural code before closing
		PreferencesUtil.setNomenclaturalCode(preferredCode);
		
		return super.close();
	}
	
	
}
