/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * Allows the user to choose which <code>Rank</code>s to display in the 
 * <code>PropertySheet</code> drop-down menu for <code>TaxonNameBase</code>.
 * 
 * @author p.ciardelli
 * @created 17.09.2008
 * @version 1.0
 */
public class RankMenuPreferences extends PreferencePage implements
		IWorkbenchPreferencePage {
	private static final Logger logger = Logger
			.getLogger(RankMenuPreferences.class);

	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.preferences.ranks";
	
	private Map<Rank, Button> rankButtons;

	public RankMenuPreferences() {
		super();
		setDescription("Choose which ranks to display in the property sheet drop-down menu.");
	}


	/**
	 * Create contents of the preference page
	 * @param parent
	 */
	@Override
	public Control createContents(Composite parent) {

		rankButtons = new HashMap<Rank, Button>();
		
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);

		SortedSet<Rank> ranks = 
				CdmSessionDataRepository.getDefault().getRanks();
		
		for (Rank rank : ranks) {
						
			Button button = new Button(container, SWT.CHECK);
			button.setText(rank.getLabel());
			rankButtons.put(rank, button);
			
			if (PreferencesUtil.getRankPreference(rank)) {
				button.setSelection(true);
			}
		}
		
		return container;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		setPreferenceStore(PreferencesUtil.getPrefStore());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		
		// Set all buttons to show feature
		for (Button button : rankButtons.values()) {
			button.setSelection(true);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	public boolean performOk() {
		
		for (Rank rank : rankButtons.keySet()) {
			Button button = rankButtons.get(rank);
			PreferencesUtil.setRankPreference(rank, button.getSelection());
		}
		
		return true;
	}
}
